@@ -51,7 +51,7 @@
 
 cmeans_clus <- function(centers.data) {
 
-  if (class(centers.data) != "centers_data")
+  if (!inherits(centers.data, "centers_data"))
     stop("centers.data must be an object of class centers_data")
 
   cClus <- lapply(centers.data, function(x) {
@@ -111,12 +111,15 @@ cmeans_clus <- function(centers.data) {
 #' @inheritParams dPCP
 #' @export
 
-plot.cmeans_clus <- function(x, ..., sample = "all") {
+plot.cmeans_clus <- function(x, ..., sample = "all", color.blind = FALSE) {
 
   if (all(sample != "all") & any(sample > length(x)))
     stop("sample must be `all` or a numeric vector indicating the row number of
          samples in sample.table")
 
+  if (!is.logical(color.blind))
+    stop("color.blind must be logical")
+
   if (all(sample == "all")) {
     plotsample <- 1:length(x)
   } else {
@@ -131,7 +134,17 @@ plot.cmeans_clus <- function(x, ..., sample = "all") {
       cluster  <-  x[[y]]$data[, ncol(x[[y]]$data) - 1]
     }
 
-    cluscolors <- scales::hue_pal()(nrow(x[[y]]$centers))
+    if (isFALSE(color.blind)) {
+      cluscolors <- scales::hue_pal()(nrow(x[[y]]$centers))
+    } else {
+      cluscolors <- c(
+        "gray70", "#004949", "#ff6db6", "#009292", "#ffb6db", "#490092",
+        "#006ddb", "#b66dff", "#6db6ff", "#b6dbff", "#920000", "#924900",
+        "#db6d00", "#24ff24", "#ffff6d",
+        "#000000")[1:(nrow(x[[y]]$centers))]
+    }
+
+
     names(cluscolors) <- row.names(x[[y]]$centers)
 
     p <- ggplot(as.data.frame(x[[y]]$data),
@@ -223,7 +236,7 @@ plot.cmeans_clus <- function(x, ..., sample = "all") {
 
 rain_reclus <- function(cmeans.cluster) {
 
-  if (class(cmeans.cluster) != "cmeans_clus")
+  if (!inherits(cmeans.cluster, "cmeans_clus"))
     stop("cmeans.cluster must be an object of class cmeans_clus")
 
   rainclus <- lapply(cmeans.cluster, function(x) {
@@ -292,12 +305,15 @@ rain_reclus <- function(cmeans.cluster) {
 #' @inheritParams dPCP
 #' @export
 
-plot.rain_reclus <- function(x, ..., sample = "all") {
+plot.rain_reclus <- function(x, ..., sample = "all", color.blind = FALSE) {
 
   if (all(sample != "all") & any(sample > length(x)))
     stop("sample must be `all` or a numeric vector indicating the row number of
          samples in sample.table")
 
+  if (!is.logical(color.blind))
+    stop("color.blind must be logical")
+
   if (all(sample == "all")) {
     plotsample <- 1:length(x)
   } else {
@@ -313,7 +329,16 @@ plot.rain_reclus <- function(x, ..., sample = "all") {
       cluster  <-  x[[y]]$data[, ncol(x[[y]]$data)]
     }
 
-    cluscolors <- scales::hue_pal()(nrow(x[[y]]$centers))
+    if (isFALSE(color.blind)) {
+      cluscolors <- scales::hue_pal()(nrow(x[[y]]$centers))
+    } else {
+      cluscolors <- c(
+        "gray70", "#004949", "#ff6db6", "#009292", "#ffb6db", "#490092",
+        "#006ddb", "#b66dff", "#6db6ff", "#b6dbff", "#920000", "#924900",
+        "#db6d00", "#24ff24", "#ffff6d",
+        "#000000")[1:nrow(x[[y]]$centers)]
+    }
+
     names(cluscolors) <- row.names(x[[y]]$centers)
 
     p <- ggplot(as.data.frame(x[[y]]$data),
@@ -363,6 +388,8 @@ plot.rain_reclus <- function(x, ..., sample = "all") {
 #'   Available formats: 'eps', 'ps', 'tex', 'pdf', 'jpeg', 'tiff', 'png',
 #'   'bmp', 'svg', 'wmf'.
 #' @param dpi numeric. Image resolution.
+#' @param color.blind logical. If TRUE colors optimized for colorblind
+#'   readers are used.
 #' @return A Shiny session.
 #' @examples
 #' \donttest{
@@ -385,11 +412,14 @@ plot.rain_reclus <- function(x, ..., sample = "all") {
 #' @import shiny
 
 manual_correction <- function(data, filename, save.plot = FALSE,
-                              format = "png", dpi = 300) {
+                              format = "png", dpi = 300, color.blind = FALSE) {
 
   if (all(class(data) != c("dPCP")))
     stop("data must be an object of class dPCP")
 
+  if (!is.logical(color.blind))
+    stop("color.blind must be logical")
+
   if (!is.character(filename) ||
       (is.character(filename) & length(filename) > 1))
     stop("filename must be a character string.")
@@ -403,10 +433,13 @@ manual_correction <- function(data, filename, save.plot = FALSE,
 
   if (!is.numeric(dpi) || length(dpi) != 1) stop("dpi must be a numeric value")
 
-  if (is.character(data$samples[[1]]$quality)) {
+  if (data$samples[[1]]$quality == "Defined by Bio-Rad") {
     partition.volume <- 0.00085
-  } else {
+  } else if (is.numeric(data$samples[[1]]$quality)) {
     partition.volume <- 0.000755
+  } else {
+    partition.volume <- as.numeric(
+      stringr::str_split(data$samples[[1]]$quality, ": ")[[1]][2])
   }
 
   clus.char <- lapply(data$samples, function(x) {
@@ -530,10 +563,25 @@ manual_correction <- function(data, filename, save.plot = FALSE,
       #Create an action button for each predited cluster
       lapply(seq(nrow(centers.data())), function(x) {
 
-        buttons.colors <- scales::hue_pal()(nrow(centers.data()))
-        background.color <- paste0(
-          "background-color:", buttons.colors[x], "; border-color: black", ";
-          font-weight: bold")
+        if (isFALSE(color.blind)) {
+          buttons.colors <- scales::hue_pal()(nrow(centers.data()))
+
+          background.color <- paste0(
+            "background-color:", buttons.colors[x], "; border-color: black", ";
+          font-weight: bold", "; color: black")
+        } else {
+          buttons.colors <- c(
+            "gray70", "#004949", "#ff6db6", "#009292", "#ffb6db", "#490092",
+            "#006ddb", "#b66dff", "#6db6ff", "#b6dbff", "#920000", "#924900",
+            "#db6d00", "#24ff24", "#ffff6d",
+            "#000000")[1:nrow(centers.data())]
+
+          background.color <- paste0(
+            "background-color:", buttons.colors[x], "; border-color: black", ";
+          font-weight: bold", "; color: grey")
+        }
+
+
 
 
         in.clus$buttons[[x]] <- actionButton(
@@ -566,7 +614,16 @@ manual_correction <- function(data, filename, save.plot = FALSE,
 
     output$finalplot <- renderPlot( {
 
-      cluscolors <- scales::hue_pal()(nrow(centers.data()))
+      if (isFALSE(color.blind)) {
+        cluscolors <- scales::hue_pal()(nrow(centers.data()))
+      } else {
+        cluscolors <- c(
+          "gray70", "#004949", "#ff6db6", "#009292", "#ffb6db", "#490092",
+          "#006ddb", "#b66dff", "#6db6ff", "#b6dbff", "#920000", "#924900",
+          "#db6d00", "#24ff24", "#ffff6d",
+          "#000000")[1:nrow(centers.data())]
+      }
+
       names(cluscolors) <- rownames(centers.data())
       ggplot(dataplot(), aes_string(x = "Vic", y = "Fam")) +
 
@@ -914,7 +971,16 @@ manual_correction <- function(data, filename, save.plot = FALSE,
 
           text.p <- ggpubr::ggparagraph(text = tx, color = "black", size = 12)
 
-          cluscolors <- scales::hue_pal()(nrow(data$samples[[x]]$centers))
+          if (isFALSE(color.blind)) {
+            cluscolors <- scales::hue_pal()(nrow(data$samples[[x]]$centers))
+          } else {
+            cluscolors <- c(
+              "gray70", "#004949", "#ff6db6", "#009292", "#ffb6db", "#490092",
+              "#006ddb", "#b66dff", "#6db6ff", "#b6dbff", "#920000", "#924900",
+              "#db6d00", "#24ff24", "#ffff6d",
+              "#000000")[1:nrow(data$samples[[x]]$centers)]
+          }
+
           names(cluscolors) <- rownames(data$samples[[x]]$centers)
 
           graph.p <- ggplot(manual.mod$clus[[x]], aes_string(x = "Vic",
